import Ember from 'ember';
import Component from '@ember/component';
import layout from '../templates/components/login-cognito-bootstrap';
import { inject as service } from '@ember/service';
import { computed } from '@ember/object';
import { set } from '@ember/object';

export default Component.extend({
  layout,
  cognito: service(),
  // Add router injection
  //router: service(),
  authentication: service(),
  username: undefined,
  password: "",
  emailInvalid: true,
  passInvalid: true,
  confirmationSent: undefined,
  confirmationCode: "",
  confirmationEmail: undefined,
  confirmed: undefined,
  passValid: computed('passValid', function () {
    if (this.get('password').length <= 6) {
      return true;
    } else {
      return false;
    }
  }),
  actions: {
    validateLoginEmail() {
      var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
      var email = this.get('username');
      if (re.test(email)) {
        this.set('emailInvalid', false);
      } else {
        this.set('emailInvalid', true);
      }
    },
    validateLoginPassword() {
      if (this.get('password').length <= 6) {
        this.set('passInvalid', true);
      } else {
        this.set('passInvalid', false);
      }
    },
    login() {
      var auth = this.get('authentication');
      this.set('authenticating', true);
      this.set('error', undefined);
      let ctrl = this,
        username = this.get('username'),
        password = this.get('password');
      if (username && password) {
        let cognito = this.get('cognito');
        cognito.authenticate(username, password)
          .then(function ( /*response*/ ) {
            ctrl.set('authenticating', false);
            Ember.Logger.debug('login with cognito succeeded.');
            let route = ctrl.get('authentication').incomingRoute
            route.transitionTo(route.routeName);
          }, function (error) {
            ctrl.set('error', error.toString().split(':')[1]);
            ctrl.set('authenticating', false);
          });
      }
    },
    resendConfirmation() {
      this.set('resentConfirmation', undefined);
      let cognito = this.get('cognito'),
        email = this.get('confirmationEmail'),
        that = this;
      cognito.resendConfirmation(email)
        .then(function (data) {
          Ember.Logger.debug(data);
          set(that, 'resentConfirmation', data);
        }, function (err) {
          Ember.Logger.debug(err);
          set(that, 'error', err);
        });
    },
    confirmRegistration() {
      var that = this;
      this.set('confirming', true);
      this.set('error', undefined);
      this.set('confirmed', undefined);
      var cognito = this.get('cognito'),
        email = this.get('confirmationEmail'),
        code = this.get('confirmationCode');
      cognito.confirm(email, code)
        .then(function (data) {
          Ember.Logger.debug(data);
          set(that, 'confirming', undefined);
          set(that, 'confirmationSent', undefined);
          set(that, 'confirmed', true);
        }, function (err) {
          Ember.Logger.debug(err);
          set(that, 'error', err);
          set(that, 'confirming', undefined);
          set(that, 'confirmed', undefined);
        });
    },
    register() {
      var that = this;
      this.set('authenticating', true);
      this.set('error', undefined);
      var password = this.get('password'),
        email = this.get('username');
      var cognito = this.get('cognito');
      cognito.register(email, password)
        .then(function (response) {
          Ember.Logger.debug(response);
          that.set('authenticating', false);
          that.set('confirmationSent', true);
          that.set('confirmationEmail', email);
        }, function (error) {
          Ember.Logger.debug(error);
          that.set('authenticating', false);
          // No confirmation sent in case of error...
          //that.set('confirmationSent', true);
          var msgArray = error.toString().split(':');
          var message = "";
          for (var i = 1; i <= msgArray.length; i++) {
            if (msgArray[i] !== undefined) {
              message += msgArray[i] + '. ';
            }
          }
          that.set('error', message);
        });
    }
  }
});
