const EOL = require('os').EOL;
const path = require('path');
const fs = require('fs');

/* eslint-env node */
module.exports = {
  description: '',

  normalizeEntityName() {}, // no argument
  
  // locals(options) {
  //   // Return custom template variables here.
  //   return {
  //     foo: options.entity.options.foo
  //   };
  // }
  fileMapTokens(options) {
    return {
      __root__() {
        if (options.inDummy) {
          return 'tests/dummy/app';
        } else {
          return 'app';
        } 
      },
      __mirage__() { return '../mirage'},
      __cloud__() { return '../cloud'}
    }
  },

  afterInstall(options) {
    // Perform extra work here.
    // Add proxy check to config/environment.js 
    let configPath = (options.dummy) ? "tests/dummy/config/environment.js" : "config/environment.js";

    let proxy = "\nfunction usingProxy() {\n"+
                    "\tvar usingProxyArg = !!process.argv.filter(function (arg) {\n"+
                    "\t\treturn arg.indexOf('--proxy') === 0 || arg.indexOf('-pr') === 0 || arg.indexOf('-pxy') === 0;\n"+
                    "\t}).length;\n\n"+
                    "\tvar hasGeneratedProxies = false;\n"+
                    "\tvar proxiesDir = process.env.PWD + '/server/proxies';\n"+
                    "\ttry {\n"+
                    "\t\tfs.lstatSync(proxiesDir);\n"+
                    "\t\thasGeneratedProxies = true;\n"+
                    "\t} catch (e) {}\n\n"+    
                    "\treturn usingProxyArg || hasGeneratedProxies;\n}\n";
    addLineToFile(this, configPath, /'use strict';/, proxy);
    addLineToFile(this, configPath, /when it is created/, "\t\tproxy: usingProxy(),");

    let validatedFormConfig = '\t\t"ember-validated-form": {\n\t\t\tlabel: {\n\t\t\t\tsubmit: "label.save"\n\t\t\t},\n'+
                              '\t\t\tcss: {\n\t\t\t\tgroup: "form-group",\n\t\t\t\tradio: "radio",\n\t\t\t\tcontrol: "form-control",\n'+
                              '\t\t\t\tlabel: "col-form-label",\n\t\t\t\thelp: "small form-text text-danger",'+
                              '\n\t\t\t\thint: "small form-text text-muted",\n\t\t\t\tcheckbox: "checkbox",\n\t\t\t\tbutton: "btn btn-default",'+
                              '\n\t\t\t\tsubmit: "btn btn-primary",\n\t\t\t\tloading: "loading",\n\t\t\t\tvalid: "is-valid",'+
                              '\n\t\t\t\terror: "is-invalid"\n\t\t\t}\n\t\t},\n\t\tgatewayURL: "",\n';
    addLineToFile(this, configPath, /let ENV = {/, validatedFormConfig);

    // Add import of ember_aws_ehipster.css to apps.css
    let stylePath = (options.dummy) ? "tests/dummy/app/styles/app.css" : "app/styles/app.css";
    let importcss = "@import 'ember-aws-ehipster.css';\n";
    let fileContents = importcss + fs.readFileSync(stylePath, 'utf-8');
    fs.writeFileSync(stylePath, fileContents, 'utf-8');

    // Add master path for dealing with entities  
    let routerPath = (options.dummy) ? "tests/dummy/app/router.js" : "app/router.js";
    let routerLine = "\tthis.route('entity-factory', function() {\n\t});\n";
    addLineToFile(this, routerPath, /Router\.map\(function\(\) {/, routerLine);

    // Add special rules to lint config file
    let lintPath = ".template-lintrc.js";
    let lintLines = ",\trules: {\n\t\t\"attribute-indentation\": false,\n\t\t\"block-indentation\": false\n\t}\n";
    //let lintLines = "\t\t\"attribute-indentation\": false,\n\t\t\"block-indentation\": false\n";
    addLineToFile(this, lintPath, /extends: 'recommended'/, lintLines);

    // Specific things for dealing with Cognito
    console.log("Complete ENV variables for the different environments");
    // Add new ENV variables
    //let configPath = require.resolve(this.project.configPath());
    let configContents = fs.readFileSync(configPath, 'utf-8');
    if (configContents.indexOf('AWS_REGION') === -1) {
      this.ui.writeLine("\tAdd AWS_REGION Env. variable to config/environnement.js");
      configContents = configContents.replace(/([ \t]+)if \(environment === 'development'\) {/, [
        '$&',
        '$1  ENV.AWS_REGION = \'<<AWS_REGION>>\';',
      ].join('\n'));
      configContents = configContents.replace(/([ \t]+)if \(environment === 'production'\) {/, [
        '$&',
        '$1  ENV.AWS_REGION = \'<<AWS_REGION>>\';',
      ].join('\n'));
    }
    if (configContents.indexOf('AWS_POOL_ID') === -1) {
      this.ui.writeLine("\tAdd AWS_POOL_ID, AWS_USER_POOL_ID and AWS_CLIENT_ID Env. variable to config/environnement.js");
      configContents = configContents.replace(/([ \t]+)if \(environment === 'development'\) {/, [
        '$&',
        '$1  ENV.AWS_POOL_ID = \'<<AWS_POOL_ID>>\';',
        '$1  ENV.AWS_USER_POOL_ID = \'<<AWS_USER_POOL_ID>>\';',
        '$1  ENV.AWS_CLIENT_ID = \'<<AWS_CLIENT_ID>>\';'
      ].join('\n'));
      configContents = configContents.replace(/([ \t]+)if \(environment === 'production'\) {/, [
        '$&',
        '$1  ENV.AWS_POOL_ID = \'<<AWS_POOL_ID>>\';',
        '$1  ENV.AWS_USER_POOL_ID = \'<<AWS_USER_POOL_ID>>\';',
        '$1  ENV.AWS_CLIENT_ID = \'<<AWS_CLIENT_ID>>\';'
      ].join('\n'));
    }

      if (configContents.indexOf('AWS_REGION') === -1 || configContents.indexOf('AWS_POOL_ID') === -1) {
        this.ui.writeWarnLine(
          `Unable to update ENV.AWS_* configuration. You should update ${configPath} to add the strings 'AWS_REGION', 'AWS_POOL_ID', 'AWS_USER_POOL_ID' and 'AWS_CLIENT_ID' in development and in production.`);
      } else {
        fs.writeFileSync(configPath, configContents, 'utf-8');
      }

    // Add Javascript libraries
    let buildContents = fs.readFileSync("ember-cli-build.js", 'utf-8');

    if (buildContents.indexOf('jsbn') === -1) {
      this.ui.writeLine("\t Add jsbn libraries to ember-cli-build.js");
      buildContents = buildContents.replace(/([ \t]+)return app\.toTree\(\);/, [
        '$1app.import\(\'vendor/jsbn.js\'\);',
        '$1app.import\(\'vendor/jsbn2.js\'\);',
        '$1app.import\(\'vendor/sjcl.js\'\);',
        '',
        '$&'
      ].join('\n'));
    }
    if (buildContents.indexOf('aws-sdk') === -1) {
      this.ui.writeLine("\tAdd Amazon SDK library to ember-cli-build.js");
      buildContents = buildContents.replace(/([ \t]+)return app\.toTree\(\);/, [
        '$1app.import\(\'vendor/aws-sdk.min.js\'\);',
        '',
        '$&'
      ].join('\n'));
    }
    if (buildContents.indexOf('amazon-cognito') === -1) {
      this.ui.writeLine("\tAdd Amazon Cognito libraries to ember-cli-build.js");
      buildContents = buildContents.replace(/([ \t]+)return app\.toTree\(\);/, [
        '$1app.import\(\'vendor/amazon-cognito/aws-cognito-sdk.min.js\'\);',
        '$1app.import\(\'vendor/amazon-cognito/amazon-cognito-identity.min.js\'\);',
        '',
        '$&'
      ].join('\n'));
    }
    if (buildContents.indexOf('amazon-cognito') === -1 || buildContents.indexOf('jsbn') === -1 || buildContents.indexOf('aws-sdk') === -1) {
      this.ui.writeWarnLine(
        `Unable to update amazon-cognito javascript libraries integration. You should update ember-cli-build to add the loading of these libraries.`);
    } else {
      fs.writeFileSync("ember-cli-build.js", buildContents, 'utf-8');
    }

     // Add route in router.js for the login page
     let routeContents = fs.readFileSync("app/router.js", 'utf-8');
     if (routeContents.indexOf('cognito-login') === -1) {
      this.ui.writeLine("\tAdd cognito-login route to router.js");
      routeContents = routeContents.replace(/Router\.map\(function\(\) {/, [
        '$&',
        '  this.route\(\'cognito-login\'\);',
        '  this.route\(\'cognito-private\'\);'
      ].join('\n'));
    }
    if (routeContents.indexOf('cognito-login') === -1) {
      this.ui.writeWarnLine(
        `Unable to update router.js. You should update this file to add the cognito-login route.`);
    } else {
      fs.writeFileSync("app/router.js", routeContents, 'utf-8');
    }  
  }
};

function addLineToFile(ctx, filePath, markerString, addedLine) {
  let fileContents = fs.readFileSync(filePath, 'utf-8');

  if (fileContents.indexOf(addedLine) === -1) {
   //ctx.ui.writeLine("\tAdd line "+addedLine+" to file "+filePath);
   fileContents = fileContents.replace(markerString, [
     '$&',
     addedLine,
   ].join('\n'));
 }
 if (fileContents.indexOf(addedLine) === -1) {
   ctx.ui.writeWarnLine(
     'Unable to update '+filePath+'. You should update this file manually.');
 } else {
   fs.writeFileSync(filePath, fileContents, 'utf-8');
 }
}
