/* eslint-env node */

const jhipster = require('jhipster-core');
const inflection = require('inflection');
const shelljs = require('shelljs');
const fs = require('fs');

module.exports = {
  name: 'jdlimporter',
  fileName: '',
  description: '',
  SEPARATOR: ';',
  entities: {},

  normalizeEntityName(entityName) {
    // Normalize and validate entity name here.
    this.fileName = entityName;
    return entityName;
  },

  beforeInstall(options) {
    this.ui.writeLine("afterInstall processing for jdlimporter module with file " + this.fileName);
    // TODO Check options contains a link on a JDL file

    // Parse provided JDL file
    var files = this.fileName.split(this.SEPARATOR);
    var jdlModel = jhipster.parseFromFiles(files);
    //this.ui.writeLine(JSON.stringify(jdlModel));

    let entities = jdlModel.entities;
    let relationships = jdlModel.relationships;

    for (var i = 0; i < entities.length; i++) {
      let entity = entities[i];
      // Add an array field to further hold options to pass to entity-factory blueprint
      entity.options = [];
      for (var j = 0; j < entity.body.length; j++) {
        let property = entity.body[j];
        let propertyType;
        switch (property.type) {
          case 'String':
          case 'TextBlob':
            propertyType = ':string';
            break;

          case 'LocalDate':
          case 'Instant':
            propertyType = ':date';
            break;

          case 'Integer':
          case 'Float':
          case 'Double':
            propertyType = ':number';
            break;

          case 'Boolean':
            propertyType = ':boolean';
            break;

          default:
            this.ui.writeLine("Huston, we have a problem with unknow type " + property.name);
            break;
        }
        let validations = property.validations;
        let validationRules = [];
        for (var k = 0; k < validations.length; k++) {
          validation = validations[k];
          let validationRule = validation.key;
          if (validation.value) {
            validationRule +='§'+validation.value;
          }
          validationRules.push(validationRule)
        }
        let validationString='';
        if (validationRules.length>0) {
          validationString = ":"+validationRules.join(',');
        }
        entity.options.push(property.name + propertyType + validationString);
      }
      // Check if current object is involved in a relationship
      // First see the 'from' relationships
      relationships.filter(relation => relation.from.name == entity.name).forEach(relation => {
        let toObjectName = relation.to.name.toLowerCase();
        // Process injectedField to better handle the relationship
        let injectedField = relation.to.injectedField;
        let field = null;
        if (injectedField != null && injectedField.indexOf('(') != -1) {
          field = injectedField.substring(injectedField.indexOf('(')+1, injectedField.indexOf(')'));
        }
        switch (relation.cardinality) {
          case "one-to-one":
          case "many-to-one":
            if (field)
              entity.options.push(toObjectName + ':belongs-to:' + toObjectName + ':'+field);
            else
              entity.options.push(toObjectName + ':belongs-to:' + toObjectName);
            break;

          case "one-to-many":
          case "many-to-many":
            if (field)
              entity.options.push(inflection.pluralize(toObjectName) + ':has-many:' + toObjectName + ':'+field);
            else
              entity.options.push(inflection.pluralize(toObjectName) + ':has-many:' + toObjectName);
            break;

          default:
            this.ui.writeLine("Huston, we have a problem with unknow relationship type " + relation.cardinality);
            break;
        }
      });
      // Do the same for to relationships
      relationships.filter(relation => relation.to.name == entity.name).forEach(relation => {
        let toObjectName = relation.from.name.toLowerCase();
        let injectedField = relation.to.injectedField;
        let field = null;
        if (injectedField != null && injectedField.indexOf('(') != -1) {
          field = injectedField.substring(injectedField.indexOf('(')+1, injectedField.indexOf(')'));
        }
        switch (relation.cardinality) {
          case "one-to-one":
          case "one-to-many":
            if (field)
              entity.options.push(toObjectName + ':has-many:' + toObjectName+ ':'+field);
            else
              entity.options.push(toObjectName + ':has-many:' + toObjectName);
            break;

          case "many-to-one":
          case "many-to-many":
            if (field)
              entity.options.push(inflection.pluralize(toObjectName) + ':has-many:' + toObjectName+ ':'+field);
            else
              entity.options.push(inflection.pluralize(toObjectName) + ':has-many:' + toObjectName);
            break;

          default:
            this.ui.writeLine("Huston, we have a problem with unknow relationship type " + relation.cardinality);
            break;
        }
      });
      let cmd = 'ember g entity-factory ' + entity.name.toLowerCase() + ' ' + entity.options.join(" ");
      if (options.dummy) {
        cmd += ' --dummy';
      }
      console.log(cmd);
      shelljs.exec(cmd);
    }

  },

  locals: function (options) {},

  afterInstall(options) {
    // Check ENV value to add AWS stuff

    // Perform extra work here.
        // Complete /mirage/config.js
        let mirageConfigPath = (options.dummy) ? "tests/dummy/mirage/config.js" : "mirage/config.js";
        let configLine = "\t\t// Enable Cognito POST\n" +
          "\t\tlet cognitoURL='https://cognito-identity.'+ENV.AWS_REGION+'.amazonaws.com/'\n" +
          "\t\tthis.passthrough(cognitoURL, ['post']);\n";
        if (!fs.existsSync(mirageConfigPath)) {
          this.ui.writeLine("Creating file " + mirageConfigPath);
          let configContent = "import ENV from './../config/environment';\n" +
            "export default function() {\n" +
            "\tthis.namespace = '';\n" +
            "\tif (!ENV.APP.proxy) {\n" + configLine + "\n\t} else {\n" +
            "\t\tthis.passthrough();\n\t}\n}\n";
          fs.writeFileSync(mirageConfigPath, configContent, 'utf-8', 'w+');
        } else {
          addLineToFile(this, mirageConfigPath, /if \(!ENV\.APP\.proxy\) {/, configLine);
        }
  }
};

function addLineToFile(ctx, filePath, markerString, addedLine) {
  let fileContents = fs.readFileSync(filePath, 'utf-8');

  if (fileContents.indexOf(addedLine) === -1) {
   //ctx.ui.writeLine("\tAdd line "+addedLine+" to file "+filePath);
   fileContents = fileContents.replace(markerString, [
     '$&',
     addedLine,
   ].join('\n'));
 }
 if (fileContents.indexOf(addedLine) === -1) {
   ctx.ui.writeWarnLine(
     'Unable to update '+filePath+'. You should update this file manually.');
 } else {
   fs.writeFileSync(filePath, fileContents, 'utf-8');
 }
}